#include <cstring>
#include <cerrno>

#include <gea/API.h>
#include <gea/gea_main.h>
#include <gea/ObjRepository.h>

#include <awds/WirelessEC.h>

using namespace awds;

int awds::WirelessEC::init()
{
    struct sockaddr_nl local;

    this->sock = socket(PF_NETLINK, SOCK_RAW, NETLINK_ROUTE);

    if (socket < 0) {
	GEA.dbg() << "Failed to open netlink socket: " << strerror(errno) << std::endl;
	return -1;
    }

    memset(&local, 0, sizeof(local));
    local.nl_family = AF_NETLINK;
    local.nl_groups = RTMGRP_LINK;
    if (bind(this->sock, (struct sockaddr *) &local, sizeof(local)) < 0) {
	GEA.dbg() << "Failed to bind netlink socket: " << strerror(errno) << std::endl;
	close();
	return -1;
    }

    handle = new gea::UnixFdHandle(sock, gea::PosixModeRead);

    GEA.waitFor(handle, gea::AbsTime::now() + gea::Duration(120),
		handle_event, this);

    return 0;
}

void awds::WirelessEC::handle_event(gea::Handle *h, gea::AbsTime t, void *data)
{
    WirelessEC *self = static_cast<WirelessEC *>(data);

    char buf[4096];
    int len;
    struct nlmsghdr *nh;
    struct ifinfomsg *ifi;
    int attrlen;
    struct rtattr *attr;

    if (h->status == gea::Handle::Ready) {
	len = recv(self->sock, buf, sizeof(buf), 0);
	for (nh = (nlmsghdr *) buf; NLMSG_OK(nh, len);
		nh = NLMSG_NEXT(nh, len)) {
	    if (nh->nlmsg_type == NLMSG_DONE)
		break;
	    if (nh->nlmsg_type == NLMSG_ERROR) {
		GEA.dbg() << "Error in netlink message." << std::endl;
		break;
	    }
	    /* From wireless.h:
	     *   Wireless events are encapsulated in the IFLA_WIRELESS field of
	     *   a RTM_NEWLINK message.
	     * So we are just interessted in that message type.
	     */
	    if (nh->nlmsg_type != RTM_NEWLINK)
		continue;

	    ifi = (ifinfomsg *) NLMSG_DATA(nh);

	    if ((self->ifindex != 0) && (self->ifindex != ifi->ifi_index))
		continue;

	    attrlen = nh->nlmsg_len - NLMSG_ALIGN(sizeof(*ifi));
	    if (attrlen > 0) {
		attr = (struct rtattr *) ((char *) ifi + NLMSG_ALIGN(sizeof(*ifi)));
		while (RTA_OK(attr, attrlen)) {
		    if (attr->rta_type == IFLA_WIRELESS) {
			self->notify((char *)RTA_DATA(attr), RTA_PAYLOAD(attr));
		    }
		    RTA_NEXT(attr, attrlen);
		}
	    }
	}
    }

    GEA.waitFor(h, t + gea::Duration(120), handle_event, data);
}


void awds::WirelessEC::notify(char *data, int len)
{
    char *pos, *end;
    struct iw_event iwe;

    pos = data;
    end = data + len;

    while(pos + IW_EV_LCP_LEN <= end) {
	memcpy(&iwe, pos, sizeof(iwe));
	GEA.dbg() << "received iwevent: cmd=0x" << std::hex << iwe.cmd
	    << " len=" << std::dec << iwe.len << std::endl;

	/* notify the registered listeners */
	if (!listeners.empty())
	    listeners(iwe);

	pos += iwe.len;
    }
}


GEA_MAIN_2(wireless_ec, argc, argv)
{
    WirelessEC *wireless_ec = new WirelessEC;

    if (wireless_ec->init() < 0) {
	GEA.dbg() << "Failed to initialise WirelessEC." << std::endl;
	return -1;
    }

    REP_INSERT_OBJ(WirelessEC *, wireless_ec, wireless_ec);

    GEA.dbg() << "WirelessEC initialised" << std::endl;

    return 0;
}
