#ifndef _WIRELESS_EC_H__
#define _WIRELESS_EC_H__

#include <sys/socket.h>
#include <linux/netlink.h>
#include <linux/rtnetlink.h>

/* linux/wireless.h results in a conflict of net/if.h and linux/if.h
 * Thus, we try to include the wireless.h from wireless tools.  */
//#include <linux/wireless.h>
#include <net/if.h>
#ifndef __user
#define __user // __user is used in wireless.h
#endif
#include <wireless.h>

#include <gea/posix/UnixFdHandle.h>

#include <awds/Callback.h>

namespace awds {
class WirelessEC {
    private:
	int sock;	///< netlink socket
	int ifindex;	///< interface index
	gea::UnixFdHandle *handle;	///< gea handle
	Callback<struct iw_event&> listeners;


	/**
	 * GEA callback to handle events.
	 */
	static void handle_event(gea::Handle *h, gea::AbsTime t, void *data);

	/**
	 * Notify the listeners about an event.
	 */
	void notify(char *data, int len);

    public:
	/**
	 * Default constructor.
	 */
	WirelessEC() : ifindex(0) { }

	/**
	 * Initialise the wireless event channel.
	 */
	int init();

	/**
	 * Close the event channel.
	 */
	void close() { ::close(sock); }

	/**
	 * Set the interface index.
	 *
	 * Only messages for the given interface index are reported.
	 */
	void setIfIndex(int ifindex) {
	    this->ifindex = ifindex;
	}

	/**
	 * Type of a callback.
	 */
	typedef bool (*we_callback)(void *, struct iw_event &);

	/**
	 * Register a new listener.
	 *
	 * The registered listeners are notified about wireless events.
	 */
	bool registerListener(we_callback cb, void *obj) {
	    this->listeners.add(cb, obj);
	}

};
}

#endif // _WIRELESS_EC_H__
